==> planqk_quantum-2.6.2-py3-none-any.whl: planqk/job.py <==
import logging
import time
from typing import Optional

from planqk.client.client import _PlanqkClient
from planqk.client.job_dtos import JobDto, JOB_STATUS, JOB_FINAL_STATES
from planqk.exceptions import PlanqkError

logger = logging.getLogger(__name__)

class PlanqkBaseJob:
    # Todo abstract backend
    def __init__(self, backend: Optional, job_id: Optional[str] = None, job_details: Optional[JobDto] = None,
                 planqk_client: Optional[_PlanqkClient] = None):
        self._client = planqk_client
        if self._client is None:
            raise RuntimeError("planqk_client must not be None")

        if job_id is None and job_details is None:
            raise ValueError("Either 'job_id', 'job_details' or both must be provided.")

        self._result = None
        self._backend = backend
        self._job_details = job_details
        self._status = None

        if job_id is not None and job_details is None:
            self._refresh(job_id)
        elif job_id is None and job_details is not None:
            self.submit()
        else:
            self._job_details = job_details

    def submit(self):
        """
        Submits the job for execution.
        """

        if self._job_details is None:
            raise RuntimeError("Cannot submit job as no job details are set.")

        self._job_details = self._client.submit_job(self._job_details)

    def result(self) -> any:
        """Polls for the job result until the job has completed successfully and returns the result.

        Returns:
            The job result if the job completed successfully. The result format is backend specific.

        Raises:
            An error if the job did reach another end state as `JOB_STATUS.COMPLETED`
        """
        if self._result is not None:
            return self._result

        status = self.status()
        if status not in JOB_FINAL_STATES:
            status = self.wait_for_final_state()

        if status == JOB_STATUS.COMPLETED:
            self._result = self._client.get_job_result(self.id)
            return self._result

        if status == JOB_STATUS.FAILED:
            if self._job_details.error_data is None:
                result_data = self._client.get_job_result(self._job_id)
            self._job_details.error_data = result_data
            raise RuntimeError(
                f'{"Cannot retrieve results as job execution failed."}'
                + f"(status: {self.status}."
                + f"error: {self.error_data})"
            )

        raise RuntimeError(
            f'{"Cannot retrieve results as job execution has not completed."}'
            + f"(status: {self.status}.")

    def wait_for_final_state(self, timeout: Optional[float] = None, wait: float = 5) -> JOB_STATUS:
        """Poll the job status until it progresses to a final state such as ``DONE`` or ``ERROR``.

        Args:
            timeout: Seconds to wait for the job. If ``None``, wait indefinitely.
            wait: Seconds between queries.
            callback: Callback function invoked after each query.
                The following positional arguments are provided to the callback function:

                * job_id: Job ID
                * job_status: Status of the job from the last query
                * job: This BaseJob instance

                Note: different subclass might provide different arguments to
                the callback function.

        Raises:
            JobTimeoutError: If the job does not reach a final state before the
                specified timeout.
        """
        start_time = time.time()
        status = PlanqkBaseJob.status(self)
        while status not in JOB_FINAL_STATES:
            logger.debug('Waiting for job %s to complete for retrieving its result. Current job status %s', self.id, status)
            elapsed_time = time.time() - start_time
            if timeout is not None and elapsed_time >= timeout:
                raise PlanqkError(f"Timeout while waiting for job {self.id}.")
            time.sleep(wait)
            status = PlanqkBaseJob.status(self) #TODO remove self_status
        return status
    
    def status(self, use_cached_value: bool = None) -> JOB_STATUS:
        """
        Return the status of the job.

        Args:
            use_cached_value (bool): If `True`, uses the value most recently retrieved from PlanQK.
        """
        if not use_cached_value or self._status not in JOB_FINAL_STATES:
            self._status = self._client.get_job_status(self.id)

        return self._status

    @property
    def id(self):
        """
        This job's id.
        """
        return self._job_details.id if self._job_details is not None else None

    def cancel(self):
        """
        Attempt to cancel the job.
        """
        self._client.cancel_job(self.id)
    
    def _refresh(self, id: str = None):
        """
        Refreshes the job details from the server.
        """
        if id is None and self.id is None:
            raise ValueError("Job Id is not set.")

        job_id = id if id is not None else self.id
        self._job_details = self._client.get_job(job_id)
        self.status()

==> planqk_quantum-2.8.0-py3-none-any.whl: planqk/job.py <==
import logging
import time
from abc import ABC
from typing import Optional

from planqk.client.client import _PlanqkClient
from planqk.client.job_dtos import JobDto, JOB_STATUS, JOB_FINAL_STATES
from planqk.exceptions import PlanqkError

logger = logging.getLogger(__name__)


class PlanqkBaseJob(ABC):
    # Todo abstract backend
    def __init__(self, backend: Optional, job_id: Optional[str] = None, job_details: Optional[JobDto] = None,
                 planqk_client: Optional[_PlanqkClient] = None):
        self._client = planqk_client
        if self._client is None:
            raise RuntimeError("planqk_client must not be None")

        if job_id is None and job_details is None:
            raise ValueError("Either 'job_id', 'job_details' or both must be provided.")

        self._result = None
        self._backend = backend
        self._job_details = job_details
        self._status = None

        if job_id is not None and job_details is None:
            self._refresh(job_id)
        elif job_id is None and job_details is not None:
            self._submit()
        else:
            self._job_details = job_details

    def _submit(self):
        """
        Submits the job for execution.
        """

        if self._job_details is None:
            raise RuntimeError("Cannot submit job as no job details are set.")

        self._job_details = self._client.submit_job(self._job_details)

    def _result(self) -> any:
        """Polls for the job result until the job has completed successfully and returns the result.

        Returns:
            The job result if the job completed successfully. The result format is backend specific.

        Raises:
            An error if the job did reach another end state as `JOB_STATUS.COMPLETED`
        """
        if self._result is not None:
            return self._result

        status = PlanqkBaseJob._status(self)
        if status not in JOB_FINAL_STATES:
            status = self._wait_for_final_state()

        if status == JOB_STATUS.COMPLETED:
            self._result = self._client.get_job_result(self.id)
            return self._result

        if status == JOB_STATUS.FAILED:
            if self._job_details.error_data is None:
                result_data = self._client.get_job_result(self.id)
            self._job_details.error_data = result_data
            raise RuntimeError(
                f'{"Cannot retrieve results as job execution failed."}'
                + f"(status: {self._status}."
                + f"error: {self.error_data})"
            )

        raise RuntimeError(
            f'{"Cannot retrieve results as job execution has not completed."}'
            + f"(status: {self._status}.")

    def _wait_for_final_state(self, timeout: Optional[float] = None, wait: float = 5) -> JOB_STATUS:
        """Poll the job status until it progresses to a final state such as ``DONE`` or ``ERROR``.

        Args:
            timeout: Seconds to wait for the job. If ``None``, wait indefinitely.
            wait: Seconds between queries.
            callback: Callback function invoked after each query.
                The following positional arguments are provided to the callback function:

                * job_id: Job ID
                * job_status: Status of the job from the last query
                * job: This BaseJob instance

                Note: different subclass might provide different arguments to
                the callback function.

        Raises:
            JobTimeoutError: If the job does not reach a final state before the
                specified timeout.
        """
        start_time = time.time()
        status = PlanqkBaseJob._status(self)
        while status not in JOB_FINAL_STATES:
            logger.debug('Waiting for job %s to complete for retrieving its result. Current job status %s', self.id, status)
            elapsed_time = time.time() - start_time
            if timeout is not None and elapsed_time >= timeout:
                raise PlanqkError(f"Timeout while waiting for job {self.id}.")
            time.sleep(wait)
            status = PlanqkBaseJob._status(self)
        return status

    def _status(self, use_cached_value: bool = False) -> JOB_STATUS:
        """
        Return the status of the job.

        Args:
            use_cached_value (bool): If `True`, uses the value most recently retrieved from PlanQK.
        """
        if not use_cached_value and self._status not in JOB_FINAL_STATES:
            self._status = self._client.get_job_status(self.id)

        return self._status

    @property
    def id(self):
        """
        This job's id.
        """
        return self._job_details.id if self._job_details is not None else None

    def cancel(self):
        """
        Attempt to cancel the job.
        """
        self._client.cancel_job(self.id)

    def _refresh(self, job_id: str = None):
        """
        Refreshes the job details from the server.
        """
        if job_id is None and self.id is None:
            raise ValueError("Job Id is not set.")

        job_id = job_id if job_id is not None else self.id
        self._job_details = self._client.get_job(job_id)
        PlanqkBaseJob._status(self)
